/*en    Laurent Constantin's network library (lcrzo)
                 lcrzo_sysdep module

  Functions herein contains system dependant functions.
*/
/*fr    Bibliotheque reseau de Laurent Constantin (lcrzo)
                 Module lcrzo_sysdep

  Les fonctions presentes dans ce module contiennent les fonctions
dependantes du systeme.
*/


/*-------------------------------------------------------------*/
#define LCRZO_PRIV_WRITEMAXMSGSIZE      500
#if defined LCRZODEF_SYSTEM_Unix
 #define LCRZO_PRIV_READMAXMSGSIZE   0x1FFFF
#elif defined LCRZODEF_SYSTEM_Windows
 /* with a bigger value, Windows doesn't have sufficient memory */
 #define LCRZO_PRIV_READMAXMSGSIZE   0x7000
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Solaris
int lcrzo_priv_dlpi_putmsg(int fd, char *ptr, int len, int flags);
int lcrzo_priv_dlpi_getmsg(int fd, int size, char *bufp);
int lcrzo_priv_dlpi_open(lcrzo_constdevice device, int *pfd);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_winreg_uint32_get(HKEY hive,
                                 lcrzo_conststring key,
                                 lcrzo_conststring name,
                                 lcrzo_uint32 *pvalue);
int lcrzo_priv_winreg_data_get(HKEY hive,
                               lcrzo_conststring key,
                               lcrzo_conststring name,
                               lcrzo_data *pvalue,
                               lcrzo_int32 *pvaluesize);
int lcrzo_priv_winreg_string_get(HKEY hive,
                                 lcrzo_conststring key,
                                 lcrzo_conststring name,
                                 lcrzo_string *pvalue);
int lcrzo_priv_winreg_stringlist_get(HKEY hive,
                                     lcrzo_conststring key,
                                     lcrzo_conststring name,
                                     lcrzo_list *plist);
int lcrzo_priv_winreg_subkeylist_get(HKEY hive,
                                     lcrzo_conststring key,
                                     lcrzo_list *plist);
int lcrzo_priv_winreg_subvallist_get(HKEY hive,
                                     lcrzo_conststring key,
                                     lcrzo_list *plist);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_dll_init(void);
int lcrzo_priv_dll_close(void);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
typedef BOOL(WINAPI *pSnmpExtensionQuery) (IN BYTE requestType, IN OUT SnmpVarBindList * variableBindings, OUT AsnInteger32 * errorStatus, OUT AsnInteger32 * errorIndex);
typedef struct {
  lcrzo_bool isloaded;
  HINSTANCE hinstance;
  pSnmpExtensionQuery snmpqueryfunction;
} lcrzo_priv_dll_snmp;
int lcrzo_priv_dll_snmp_init(lcrzo_priv_dll_snmp *pdll_snmp);
int lcrzo_priv_dll_snmp_close(lcrzo_priv_dll_snmp *pdll_snmp);
extern lcrzo_priv_dll_snmp lcrzo_global_dll_snmp;
int lcrzo_priv_dll_snmpoid_init(AsnObjectIdentifier *DstObjId, ...);
int lcrzo_priv_dll_snmpoid_close(AsnObjectIdentifier *ObjId);
int lcrzo_priv_dll_snmpoid_print(AsnObjectIdentifier *ObjId);
typedef struct {
  SnmpVarBindList varBindList;
  SnmpVarBind varBind[5];
} lcrzo_priv_dll_snmpreq;
int lcrzo_priv_dll_snmpreq_init(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                               AsnObjectIdentifier *poid1,
                               AsnObjectIdentifier *poid2,
                               AsnObjectIdentifier *poid3,
                               AsnObjectIdentifier *poid4,
                               AsnObjectIdentifier *poid5);
int lcrzo_priv_dll_snmpreq_close(lcrzo_priv_dll_snmpreq *pdll_snmpreq);
int lcrzo_priv_dll_snmpreq_print(lcrzo_priv_dll_snmpreq *pdll_snmpreq);
int lcrzo_priv_dll_snmpreq__uint32(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                                  lcrzo_uint8 index,
                                  lcrzo_uint32 *pnumber);
int lcrzo_priv_dll_snmpreq__data(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                                lcrzo_uint8 index,
                                lcrzo_data *pdata,
                                lcrzo_int32 *pdatasize);
int lcrzo_priv_dll_snmpreq__etha(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                                lcrzo_uint8 index,
                                lcrzo_etha etha);
int lcrzo_priv_dll_snmpreq_getnext(lcrzo_priv_dll_snmpreq *pdll_snmpreq);
int lcrzo_priv_dll_snmp_printall(void);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
typedef DWORD (WINAPI*pGetIpAddrTable) (PMIB_IPADDRTABLE pIpAddrTable, PULONG pdwSize, BOOL bOrder);
typedef DWORD (WINAPI*pGetIpNetTable) (PMIB_IPNETTABLE pIpNetTable, PULONG pdwSize, BOOL bOrder);
typedef DWORD (WINAPI*pGetIpForwardTable) (PMIB_IPFORWARDTABLE pIpForwardTable, PULONG pdwSize, BOOL bOrder);
typedef DWORD (WINAPI*pGetIfEntry) (PMIB_IFROW pIfRow);
typedef DWORD (WINAPI*pSendARP) (IPAddr DestIP, IPAddr SrcIP, PULONG pMacAddr, PULONG PhyAddrLen);
typedef DWORD (WINAPI*pGetIfTable) (PMIB_IFTABLE pIfTable, PULONG pdwSize, BOOL bOrder);
typedef struct {
  lcrzo_bool isloaded;
  HINSTANCE hinstance;
  pGetIpAddrTable GetIpAddrTable;
  pGetIpNetTable GetIpNetTable;
  pGetIpForwardTable GetIpForwardTable;
  pGetIfEntry GetIfEntry;
  pSendARP SendARP;
  pGetIfTable GetIfTable;
} lcrzo_priv_dll_iphlpapi;
int lcrzo_priv_dll_iphlpapi_init(lcrzo_priv_dll_iphlpapi *pdll_iphlpapi);
int lcrzo_priv_dll_iphlpapi_close(lcrzo_priv_dll_iphlpapi *pdll_iphlpapi);
extern lcrzo_priv_dll_iphlpapi lcrzo_global_dll_iphlpapi;
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
typedef int (WINAPI*pWSAStartup) (WORD wVersionRequested, LPWSADATA lpWSAData);
typedef int (WINAPI*pWSACleanup) (void);
typedef SOCKET (WINAPI*psocket) (int af, int type, int protocol);
typedef int (WINAPI*pclosesocket) (SOCKET s);
typedef int (WINAPI*psetsockopt) (SOCKET s, int level, int optname, const char FAR *optval, int optlen);
typedef int (WINAPI*psendto) (SOCKET s, const char FAR *buf, int len, int flags, const struct sockaddr FAR *to, int tolen);
typedef struct {
  lcrzo_bool isloaded;
  HINSTANCE hinstance;
  pWSAStartup WSAStartup;
  pWSACleanup WSACleanup;
  psocket socket;
  pclosesocket closesocket;
  psetsockopt setsockopt;
  psendto sendto;
} lcrzo_priv_dll_winsock2;
int lcrzo_priv_dll_winsock2_needtoload(lcrzo_bool *pyes);
int lcrzo_priv_dll_winsock2_init(lcrzo_priv_dll_winsock2 *pdll_winsock2);
int lcrzo_priv_dll_winsock2_close(lcrzo_priv_dll_winsock2 *pdll_winsock2);
extern lcrzo_priv_dll_winsock2 lcrzo_global_dll_winsock2;
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
typedef DWORD (WINAPI*pRasEnumConnections) (LPRASCONN lprasconn, LPDWORD lpcb, LPDWORD lpcConnections);
typedef DWORD (WINAPI*pRasGetProjectionInfo) (HRASCONN hrasconn, RASPROJECTION rasprojection, LPVOID lpprojection, LPDWORD lpcb);
typedef struct {
  lcrzo_bool isloaded;
  HINSTANCE hinstance;
  pRasEnumConnections RasEnumConnections;
  pRasGetProjectionInfo RasGetProjectionInfo;
} lcrzo_priv_dll_rasapi32;
int lcrzo_priv_dll_rasapi32_init(lcrzo_priv_dll_rasapi32 *pdll_rasapi32);
int lcrzo_priv_dll_rasapi32_close(lcrzo_priv_dll_rasapi32 *pdll_rasapi32);
extern lcrzo_priv_dll_rasapi32 lcrzo_global_dll_rasapi32;
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Unix
int lcrzo_priv_fd_block_set(int fd, 
			    lcrzo_bool beblocking);
int lcrzo_priv_fd_block_get(int fd, 
			    lcrzo_bool *pbeblocking);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_winsock_init(void);
int lcrzo_priv_winsock_close(void);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_wait_winmsec_init_time(lcrzo_time *ptime, 
				      lcrzo_uint32 *pmsec);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_criticalsection_init(void);
int lcrzo_priv_criticalsection_close(void);
int lcrzo_priv_cs_PacketGetAdapterNames(PTSTR pStr, 
                                        PULONG BufferSize);
int lcrzo_priv_cs_PacketOpenAdapter(LPTSTR AdapterName,
                                    LPADAPTER *plp);
int lcrzo_priv_cs_pcap_open_live(char *dev, 
                                 int mtu, 
                                 int promisc, 
                                 int toms, 
                                 char *err,
                                 pcap_t* *ppc);
int lcrzo_priv_cs_PacketCloseAdapter(LPADAPTER lpAdapter);
int lcrzo_priv_cs_pcap_close(pcap_t *pt);
#endif

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_winpcap_version(lcrzo_uint32 *pdrivermajor,
                               lcrzo_uint32 *pdriverminor);
#endif

/*-------------------------------------------------------------*/
int lcrzo_priv_spoof_supports_eth(lcrzo_constdevice device,
				  lcrzo_bool *pyes);
int lcrzo_priv_spoof_supports_ip(lcrzo_bool *pyes);

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_wait(int fd, 
		       lcrzo_time *pmaxwaitingtime, 
		       lcrzo_bool *pevent);
int lcrzo_priv_wait_fd_wait(int fd,
			    lcrzo_time *pmaxwaitingtime, 
			    lcrzo_bool *pevent);
int lcrzo_priv_wait_fd_read(int fd,
                            lcrzo_data *pstoreddata,
                            lcrzo_int32 *pstoreddatasize);

/*-------------------------------------------------------------*/
int lcrzo_priv_program_exit(void);

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_init(void);
int lcrzo_priv_unix_signalhandler_firstprocess_init(void);
int lcrzo_priv_unix_signalhandler_firstprocess_kill(void);
int lcrzo_priv_unix_signalhandler_child_init(void);
int lcrzo_priv_unix_signalhandler_child_add(lcrzo_uint32 processnumber);
int lcrzo_priv_unix_signalhandler_child_del(lcrzo_uint32 processnumber);
int lcrzo_priv_unix_signalhandler_child_killall(void);
int lcrzo_priv_unix_signalhandler_set(int signumber, 
				      lcrzo_unix_signalhandler sh);
int lcrzo_priv_unix_signalhandler_get(int signumber, 
				      lcrzo_unix_signalhandler *psh);

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_HANDLE_TYPE_UNKNOWN 1
#define LCRZO_PRIV_HANDLE_TYPE_CONSOLE 2
#define LCRZO_PRIV_HANDLE_TYPE_FILE    3
#define LCRZO_PRIV_HANDLE_TYPE_PIPE    4
int lcrzo_priv_wait_handle_wait(LCRZOHANDLE thehandle,
				lcrzo_int16 handletype,
				lcrzo_time *pmaxwaitingtime, 
				lcrzo_bool *pevent);


